package Arjen;
use strict;
use warnings;
use ArjenDB;
use File::Basename;
use Cwd qw/abs_path/;
use Digest::SHA1 qw/sha1_hex/;

BEGIN {
    use Exporter ();
    use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

    @ISA         = qw(Exporter);
    @EXPORT      = qw(checkFile checkSource);
    %EXPORT_TAGS = ();
    @EXPORT_OK   = qw();
}

use vars @EXPORT_OK;
END { }    # module clean-up code here (global destructor)

sub checkFile {
    return if @_ < 1;
    my $fh = shift;
    return checkSource ( join ( "", <$fh> ), @_ );
}

sub checkSource {
    return if @_ < 1;
    my $source = shift;
    my $utoken = undef;
    if ( @_ >= 1 ) {
        $utoken = shift;
        if ( $utoken ) {
            $utoken =~ s/(^\s*|\s*$)//;
            $utoken = undef if $utoken eq "";
        }
        $utoken = undef if not $utoken;
        $utoken = sha1_hex ( $utoken ) if $utoken;
    }
    my $storeSource = ( @_ >= 1 ? shift : undef );
    my $ip = ( @_ >= 1 ? shift : undef );

    # Start retrieving errors
    my @errors = ();
    my $dirname = abs_path ( dirname ( __FILE__ ) );
    opendir my $BACKENDS, $dirname . '/backends/' or die "Could not open backends directory: $!\n";
    while ( $_ = readdir $BACKENDS ) {
        next if not /\.pm$/;
        require "backends/$_";
        s/\..*//;
        eval sprintf ( '@errors = ( @errors, @{ %s::checkSources ( $source ) } );', $_ );
    }
    closedir $BACKENDS;

    # Sort the errors by line number
    @errors = sort { $a -> { "line" } <=> $b -> { "line" } } @errors;

    ArjenDB::h -> do ( "INSERT INTO checks ( user_id, source, ip, date ) VALUES ( ?, ?, ?, datetime('now','localtime') )", undef, $utoken, $storeSource ? $source : undef, $ip ) or die "Fatal DB error: $@";
    my $checkId = ArjenDB::h -> last_insert_id ( undef, undef, 'checks', 'id' );

    my $sth = ArjenDB::h -> prepare ( "INSERT INTO check_errors ( check_id, error_id, source_line, date ) VALUES ( ?, ?, ?, datetime('now','localtime') )" );
    foreach my $error ( @errors ) {
        next if not defined $error -> { "error_id" };
        $sth -> execute ( $checkId, $error -> { "error_id" }, $error -> { "line" } );
    }

    my $errorDescriptions = ArjenDB::h -> selectall_hashref ( "SELECT * FROM errors", "id" );

    # We don't want to return unknown errors...
    # wkt: Also skip errors whose title starts with "skip"
    @errors = grep {
	   defined $_ ->{"error_id"} &&
	     !($errorDescriptions->{ $_->{"error_id"} }->{error} =~ /^skip/)
	} @errors;

    foreach my $error ( @errors ) {
        my $errorDescription = $errorDescriptions -> { $error -> { "error_id" } };
        $error -> { "error" } = $errorDescription -> { "error" };
        $error -> { "description" } = sprintf $errorDescription -> { "description" }, @{ $error -> { "more" } };
    }

    return \@errors;
}

1;
