package Errors;
use strict;
use warnings;
require 'ArjenDB.pm';

BEGIN {
    use Exporter ();
    use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

    @ISA         = qw(Exporter);
    @EXPORT      = qw(addError getErrors);
    %EXPORT_TAGS = ();
    @EXPORT_OK   = qw();
}

### GLOBALS ###
# Hash of known errors, keyed by error-id. The value is a hashref with
# two indexes: error and description (multiline).
my %errorList;
# List of errors
my @errors;

use vars @EXPORT_OK;

# addError: Given the line number, the error-id and an optional list
# of information strings, add this as an error to the list of
# errors found.
sub addError {
    push( @errors, [@_] );
}

# Read in the list of known errors
sub readKnownErrors {
    if ( not scalar keys %errorList ) {
        %errorList = %{ ArjenDB::h -> selectall_hashref ( "SELECT * FROM errors_yacc", "etoken" ) };
    }
}

# Returns a list of errors found during the parsing of the given source file
# The returned list contains hashrefs, each with the following indexes:
# line => The line number the error occured on
# title => A short, one-line error message
# description => A longer, HTML formatted description of the error
sub getErrors {
    readKnownErrors;

    my @builtErrorList = ();
    # Build a suitable structure for output
    # Each error is an arrayref of at least 2 values: linenumber and errorid.
    # This can be followed by a list of strings which will be inserted
    # into the error description using sprintf markup.
    foreach my $eref ( @errors ) {
        my $line   = shift @{$eref};
        my $etoken = shift @{$eref};

        if ( not defined $errorList { $etoken } ) {
            # These are usually caught by javac
            next if $etoken eq "unexpected_token";

            print STDERR "Unknown error: '$etoken'\n";
            print STDERR "\ton line $line\n";
            next;
        }

        # Let sprintf handle various extra pieces of information used in description
        my $eid = $errorList { $etoken } -> { "error_id" };

        # Build error list
        push @builtErrorList, { line => $line, error_id => $eid, source => undef, more => $eref };
    }

    return \@builtErrorList;
}

1;
