package Variables;
use strict;
use warnings;

BEGIN {
    use Exporter ();
    use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

    @ISA         = qw(Exporter);
    @EXPORT      = qw(findVar addVar addScope delScope);
    %EXPORT_TAGS = ();
    @EXPORT_OK   = qw();
}

use vars @EXPORT_OK;
END { }    # module clean-up code here (global destructor)

### GLOBALS ###
my @Scope;       # List of all scopes in this file
my $Curscope;    # Current scope with refs to all the variables in it

# addScope: start an empty new scope. The argument is the line number.
sub addScope {
    my ($lnum) = @_;

    # Create an empty hash and point Curscope at it.
    $Curscope = {};

    # Push this new scope onto the scope list
    push( @Scope, $Curscope );

    #print("After AddScope line $lnum, we have\n");
    #print Dumper(\@Scope);
}

# delScope: remove a scope and all the variables in it.
# The argument is the line number.
sub delScope {
    my ($lnum) = @_;
    pop(@Scope);    # Lose the current scope
    $Curscope = $Scope[-1];    # Point $Curscope at the one before it
    #print("After DelScope line $lnum, we have\n");
    #print Dumper(\@Scope);
}

# addVar: given a name, type, linenumber and list of modifiers, make a new
# hash in the current scope
sub addVar {
    my ( $name, $type, $line, $modifiers ) = @_;

    # First check if we are clashing with a variable of the same name
    my $top = @Scope - 1;
    for ( my $level = $top ; $level >= 0 ; $level-- ) {

        # Get the scope at this level
        my $s = $Scope[$level];
    }

    # Now add the variable to the current scope
    $Curscope->{$name}{type}      = $type;
    $Curscope->{$name}{line}      = $line;
    $Curscope->{$name}{modifiers} = $modifiers if ( defined($modifiers) );

    #print("After adding $name on line $line, we have\n");
    #print Dumper( \@Scope );
}

# findVar: given a variable name, return a hashref to it plus
# a number indicating the scope: 0 is global, 1 is next down etc.
# Also return the current scope number. An undef hashref means no
# variable found.
sub findVar {
    my ($name) = @_;

    # Search from the innermost scope outwards
    my $top = @Scope - 1;
    for ( my $level = $top ; $level >= 0 ; $level-- ) {

        # Get the scope at this level
        my $s = $Scope[$level];

        if ( defined( $s->{$name} ) ) {
            return ( $s->{$name}, $level, $top );
        }
    }

    # Did not find it, return undef
    return (undef);
}

1;
