import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.net.MalformedURLException;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.net.URLConnection;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import bluej.extensions.BClass;
import bluej.extensions.BlueJ;
import bluej.extensions.Extension;
import bluej.extensions.MenuGenerator;
import bluej.extensions.PreferenceGenerator;
import bluej.extensions.editor.Editor;
import bluej.extensions.editor.TextLocation;
import bluej.extensions.event.PackageEvent;
import bluej.extensions.event.PackageListener;

/**
 * Start of Arjen BlueJ extension
 * 
 * Build with : javac -cp C:\bluej\lib\bluejext.jar;. ArjenExtension.java & jar cvfe arjen.jar ArjenExtension *.class & copy /Y arjen.jar C:\bluej\lib\extensions\
 * or equivalent
 * 
 * @author Jon Gjengset
 */
public class ArjenExtension extends Extension implements PackageListener {
    public static final String UID_DEFAULT = "";
    public static final String URL_DEFAULT = "http://your.webserver.edu/cgi-bin/arjen.pl2";
    public static final boolean SOURCE_DEFAULT = true;
    public static final String UID_LABEL = "UID";
    public static final String URL_LABEL = "URL";
    public static final String SOURCE_LABEL = "SOURCE";

    /**
     * Initialises the extension by setting the generator for menu items
     */
    public void startup ( BlueJ bluej ) {
        Preferences myPreferences = new Preferences ( bluej );
        bluej.setPreferenceGenerator ( myPreferences );

        ArjenMenuBuilder amb = new ArjenMenuBuilder ( bluej );
        bluej.setMenuGenerator ( amb );
    }

    /**
     * This method must decide if this Extension is compatible with the
     * current release of the BlueJ Extensions API
     */
    public boolean isCompatible ( ) {
        return true;
    }

    /**
     * Returns the version number of this extension
     */
    public String getVersion ( ) {
        return ( "2010.09" );
    }

    /**
     * Returns the user-visible name of this extension
     */
    public String getName ( ) {
        return ( "Arjen Extension" );
    }

    /**
     * Returns a simple description of the extension
     */
    public String getDescription ( ) {
        return ( "Extension for checking code for errors using Arjen" );
    }

    /**
     * Returns a URL where you can find info on this extension.
     * TODO: Set a proper URL
     */
    public URL getURL ( ) {
        try {
            return new URL ( "http://www.google.com" );
        } catch ( MalformedURLException e ) {
        }
        return null;
    }

    @Override
    public void packageOpened ( PackageEvent event ) {
    }

    @Override
    public void packageClosing ( PackageEvent event ) {
    }
}

/**
 * Settings dialog for Arjen
 */
class Preferences implements PreferenceGenerator {
    private JPanel panel;
    private JTextField uid;
    private JTextField url;
    private JCheckBox source;
    private BlueJ bluej;

    // Construct the panel, and initialise it from any stored values
    public Preferences ( BlueJ bluej ) {
        this.bluej = bluej;

        this.panel = new JPanel ( );
        this.panel.setBorder ( BorderFactory.createEmptyBorder ( 10, 10, 10, 10 ) );
        this.panel.setLayout ( new GridLayout ( 0, 2 ) );

        this.panel.add ( new JLabel ( "User ID:" ) );
        this.uid = new JTextField ( 40 );
        this.panel.add ( uid );

        this.panel.add ( new JLabel ( "Arjen server:" ) );
        this.url = new JTextField ( 40 );
        this.panel.add ( url );

        this.panel.add ( new JLabel ( "Send source(s):" ) );
        this.source = new JCheckBox ( );
        this.panel.add ( source );

        // Load the default values
        this.loadValues ( );
    }

    public JPanel getPanel ( ) {
        return this.panel;
    }

    public void saveValues ( ) {
        // Save all preferences to the BlueJ config file
        this.bluej.setExtensionPropertyString ( ArjenExtension.UID_LABEL, this.uid.getText ( ) );
        this.bluej.setExtensionPropertyString ( ArjenExtension.URL_LABEL, this.url.getText ( ) );
        this.bluej.setExtensionPropertyString ( ArjenExtension.SOURCE_LABEL, Boolean.toString ( this.source.isSelected ( ) ) );
    }

    public void loadValues ( ) {
        // Load values from settings with "sensible" defaults
        this.uid.setText ( this.bluej.getExtensionPropertyString ( ArjenExtension.UID_LABEL, ArjenExtension.UID_DEFAULT ) );
        this.url.setText ( this.bluej.getExtensionPropertyString ( ArjenExtension.URL_LABEL, ArjenExtension.URL_DEFAULT ) );
        this.source.setSelected ( Boolean.parseBoolean ( this.bluej.getExtensionPropertyString ( ArjenExtension.SOURCE_LABEL, Boolean.toString ( ArjenExtension.SOURCE_DEFAULT ) ) ) );
    }
}

/**
 * Menu builder for Arjen
 * 
 * @author Jon Gjengset
 */
class ArjenMenuBuilder extends MenuGenerator {
    /**
     * The currently open class file
     */
    private BClass currentClass;

    /**
     * Running instance of BlueJ
     */
    final public BlueJ bluej;

    public ArjenMenuBuilder ( BlueJ bluej ) {
        this.bluej = bluej;
    }

    /**
     * Returns the item to be added to the tools menu
     */
    public JMenuItem getClassMenuItem ( BClass aClass ) {
        return new JMenuItem ( new ArjenAction ( ) );
    }

    /**
     * Notification on class change
     */
    public void notifyPostClassMenu ( BClass bc, JMenuItem jmi ) {
        this.currentClass = bc;
    }

    /**
     * Makes a call to Arjen to check the currently open class file
     * Shows a popup with details about errors
     */
    private void callArjen ( ) {
        Editor classEditor = null;
        try {
            classEditor = this.currentClass.getEditor ( );
        } catch ( Exception e ) {
        }

        if ( classEditor == null ) {
            System.out.println ( "Can't create Editor for " + this.currentClass );
            return;
        }

        TextLocation firstChar = classEditor.getTextLocationFromOffset ( 0 );
        TextLocation lastChar = classEditor.getTextLocationFromOffset ( classEditor.getTextLength ( ) );
        final String source = classEditor.getText ( firstChar, lastChar );

        /**
         * Show progress dialog
         */
        Frame f = null;
        final JDialog progress = new JDialog ( f, "Calling Arjen" );
        JProgressBar dpb = new JProgressBar ( );
        dpb.setIndeterminate ( true );
        progress.add ( BorderLayout.CENTER, dpb );
        JLabel txt = new JLabel ( "Exchanging data with Arjen..." );
        txt.setBorder ( BorderFactory.createEmptyBorder ( 10, 10, 10, 10 ) );
        progress.add ( BorderLayout.NORTH, txt );
        progress.setDefaultCloseOperation ( JDialog.DO_NOTHING_ON_CLOSE );
        progress.setSize ( 300, 95 );
        progress.setLocationRelativeTo ( null );
        progress.setVisible ( true );

        ( new Thread ( ) {
            public void run ( ) {
                /**
                 * Here, we have to call Arjen
                 * Links of interest:
                 * - http://download-llnw.oracle.com/javase/1.4.2/docs/api/javax/xml/parsers/DocumentBuilder.html#
                 * DocumentBuilder()
                 * - http://www.rgagnon.com/javadetails/java-0408.html
                 * - http://snippets.dzone.com/posts/show/6984
                 */
                try {
                    String uid = ArjenMenuBuilder.this.bluej.getExtensionPropertyString ( ArjenExtension.UID_LABEL, ArjenExtension.UID_DEFAULT );
                    String hosturl = ArjenMenuBuilder.this.bluej.getExtensionPropertyString ( ArjenExtension.URL_LABEL, ArjenExtension.URL_DEFAULT );
                    boolean sendSource = Boolean.parseBoolean ( ArjenMenuBuilder.this.bluej.getExtensionPropertyString ( ArjenExtension.SOURCE_LABEL, Boolean.toString ( ArjenExtension.SOURCE_DEFAULT ) ) );

                    while ( uid == null || uid.trim ( ).isEmpty ( ) ) {
                        uid = JOptionPane.showInputDialog ( "You have not yet set a user ID, please input your user ID in the field below and press OK", System.getProperty ( "user.name" ) );
                    }

                    ArjenMenuBuilder.this.bluej.setExtensionPropertyString ( ArjenExtension.UID_LABEL, uid );

                    URL url = new URL ( hosturl + "?user=" + uid + ( sendSource ? "&source=1" : "" ) );
                    URLConnection con = url.openConnection ( );
                    con.setConnectTimeout ( 10 * 1000 );
                    con.setReadTimeout ( 10 * 1000 );

                    // specify that we will send output and accept input
                    con.setDoInput ( true );
                    con.setDoOutput ( true );

                    con.setUseCaches ( false );
                    con.setDefaultUseCaches ( false );

                    // tell the web server what we are sending
                    con.setRequestProperty ( "Content-Type", "text/plain" );

                    OutputStreamWriter writer = new OutputStreamWriter ( con.getOutputStream ( ) );
                    writer.write ( source );
                    writer.flush ( );
                    writer.close ( );

                    DocumentBuilder db = DocumentBuilderFactory.newInstance ( ).newDocumentBuilder ( );
                    Document dom = db.parse ( con.getInputStream ( ) );

                    NodeList errors = dom.getElementsByTagName ( "error" );

                    // No errors = success, so make the dialog smaller and show "No errors found"
                    if ( errors.getLength ( ) == 0 ) {
                        progress.setVisible ( false );
                        progress.dispose ( );
                        JOptionPane.showMessageDialog ( null, "No errors found!" );
                        return;
                    }

                    String output = "";
                    for ( int i = 0; i < errors.getLength ( ); i++ ) {
                        Element error = (Element) errors.item ( i );
                        output += "<h3>";
                        output += "<strong>";
                        output += "Line ";
                        output += error.getAttribute ( "line" );
                        output += ":&nbsp;";
                        output += error.getAttribute ( "title" );
                        output += "</h3>";
                        output += error.getElementsByTagName ( "description" ).item ( 0 ).getTextContent ( );
                        output += "</div>";
                    }

                    // Now let's create a dialog!
                    Window w = null;
                    final JDialog dialog = new JDialog ( w, "Arjen results" );

                    // Add contents to it.
                    JEditorPane errorPane = new JEditorPane ( "text/html", "<html>" + output + "</html>" );
                    errorPane.setFont ( errorPane.getFont ( ).deriveFont ( Font.PLAIN, 14.0f ) );
                    errorPane.setEditable ( false );

                    // Start scrolling from the top
                    errorPane.setCaretPosition ( 0 );

                    // But we also want the panel to be scrollable:
                    JScrollPane scrollPane = new JScrollPane ( errorPane );
                    scrollPane.setHorizontalScrollBarPolicy ( JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );

                    // And a close button
                    JButton closeButton = new JButton ( "Close" );
                    closeButton.addActionListener ( new ActionListener ( ) {
                        public void actionPerformed ( ActionEvent e ) {
                            dialog.setVisible ( false );
                            dialog.dispose ( );
                        }
                    } );
                    closeButton.setAlignmentX ( Component.CENTER_ALIGNMENT );
                    closeButton.setFont ( closeButton.getFont ( ).deriveFont ( Font.PLAIN, 15.0f ) );
                    JPanel closePanel = new JPanel ( );
                    closePanel.setLayout ( new BoxLayout ( closePanel, BoxLayout.Y_AXIS ) );
                    closePanel.setBorder ( BorderFactory.createEmptyBorder ( 5, 5, 5, 5 ) );
                    closePanel.add ( closeButton );

                    JPanel contentPane = new JPanel ( new BorderLayout ( ) );
                    contentPane.add ( scrollPane, BorderLayout.CENTER );
                    contentPane.add ( closePanel, BorderLayout.PAGE_END );
                    contentPane.setOpaque ( true );
                    dialog.setContentPane ( contentPane );

                    dialog.setSize ( new Dimension ( 700, 500 ) );
                    dialog.setLocationRelativeTo ( null );
                    
                    // Remove progress and show results
                    progress.setVisible ( false );
                    progress.dispose ( );
                    dialog.setVisible ( true );
                } catch ( SocketTimeoutException t ) {
                    JOptionPane.showMessageDialog ( null, "The connection to Arjen timed out!" );
                } catch ( IOException t ) {
                    JOptionPane.showMessageDialog ( null, "Could not access Arjen!" );
                } catch ( Throwable t ) {
                    JOptionPane.showMessageDialog ( null, "An unexpected error occured when contacting Arjen: " + t.getMessage ( ) );
                }
                
                progress.setVisible ( false );
                progress.dispose ( );
            }
        } ).start ( );
        
    }

    /**
     * Action handler for the menu item
     * 
     * @author Jon Gjengset
     */
    class ArjenAction extends AbstractAction {
        public ArjenAction ( ) {
            putValue ( AbstractAction.NAME, "Check with Arjen" );
        }

        public void actionPerformed ( ActionEvent e ) {
            callArjen ( );
        }
    }
}
